#  - converter.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""Process a list of files.

	Do the conversion. Pass eatlist an Options instance."""

import sys
import os
import ogg.vorbis
import ID3


# iterate through the playlist 
def eatlist(opts, playlist):
	"""Reencode Files.

	Re-encodes a list of files into a 
	portable friendly format. Ignores 
	nonexistant files. Understands xmms
	'.pls' playlists and .m3u files."""

	opts.log(1, "Writing files to %s" % opts['playerdir'])
	bytecount = 0
	for filename in playlist:
		if not os.path.exists(filename):
			opts.log(2, "\"%s\" is not a valid file" % filename)
			continue

		#cook up some filenames
		pathto, outname = os.path.split(filename)
		outname, extension = os.path.splitext(outname)
		extension = extension.lower()
		outname = outname + opts['brwarning']  
		outname = outname.replace(' ', '_').replace("'", "")
		outname = outname.replace(':', '').replace("?", "")
		outname = outname.replace('*', '')
		outname = outname.replace('`', '')
		wavname = os.path.join(opts['tempdir'], outname) + ".wav"
		if opts['treedepth']:
			pathto, dir = os.path.split(pathto)
			for i in range(opts['treedepth'] -1):
				pathto, e = os.path.split(pathto)
				dir = os.path.join(e, dir)
		else:
			dir = ''
		mp3name = os.path.join(opts['playerdir'], dir)
		mp3name = mp3name.replace(':', '') # lyra chokes on ':'
		if not os.path.isdir(mp3name):
			os.makedirs(mp3name)
		indexname = os.path.join(mp3name, "track.index")
		mp3name = os.path.join(mp3name, outname)
		if opts['encoder'] == 'lame':
			mp3name += ".mp3"
		elif opts['encoder'] == 'oggenc':
			mp3name += ".ogg"
		elif opts['encoder'] == 'wave':
			mp3name += ".wav"

		#set up decoder
		if opts['encoder'] == 'wave':
			wavname = mp3name  # decode right to the output file in wave mode
		if extension == ".ogg":
			if not opts.bin['ogg123']:
				opts.log(1, "'ogg123' binary not found! Aborting track.")
				continue
			decodecmd = "%s --device wav --file \"%s\" \"%s\" " % \
			  (opts.bin['ogg123'], wavname, filename)
			tags = ogg.vorbis.VorbisFile(filename).comment().as_dict()
		elif extension == ".mp3":
			if not opts.bin['mpg321']:
				opts.log(1, "'mpg321' binary not found! Aborting track.")
				continue
			decodecmd = "%s --wav \"%s\" \"%s\" " % \
			  (opts.bin['mpg321'], wavname, filename)
			tags = ID3.ID3(filename, as_tuple=1).as_dict()
		elif extension == ".flac":
			if not opts.bin['flac']:
				opts.log(1, "'flac' binary not found! Aborting track.")
				continue
			decodecmd = '%s --decode -F -o "%s" "%s" ' %\
			  (opts.bin['flac'], wavname, filename)
			tags = {}
			if opts.bin['metaflac']:
				cmd = '%s --export-tags-to=- "%s" ' % (opts.bin['metaflac'], filename)
				fd = os.popen(cmd)
				data = fd.read()
				fd.close()
				data = data.split('\n')
				for tag in data:
					pair = tag.split('=', 1)
					if len(pair) == 2:
						tags[pair[0].upper()] = (pair[1],)
		elif extension == ".wav":
			# make a copy so we don't normalize the original
			decodecmd = 'cp "%s" "%s" ' % (filename, wavname)
			tags = {}
		else:
			#ignore files we don't understand
			opts.log(2, "Ignoring: %s" % filename)
			continue
			

		#guess missing tags
		name = os.path.splitext(filename)[0].replace('_', ' ')
		for id, depth in [('ARTIST', -3), ('ALBUM', -2), ('TITLE', -1)]:
			if not id in tags or tags[id][0] == '':
				try:
					tags[id] = [name.split(os.sep)[depth]]
					if tags[id][0][-4:].lower() in [".ogg", ".mp3"]:
						tags[id][0] = tags[id][0][:-4]
				except IndexError:
					tags[id] = "%s unknown" % id.lower()
			tags[id] = [tags[id][0].replace('`', '\`')]
			#tags[id][0] = tags[id][0].replace('*', '')
			#tags[id][0] = tags[id][0].replace(':', '')

		#set up encoder
		if opts['encoder'] == 'lame':
			if not opts.bin['lame']:
				opts.log(1, "'lame' binary not found! Aborting track.")
				continue
			encodecmd = "%s %s \"%s\" \"%s\"" % \
			  (opts.bin['lame'], opts['encopts'], wavname, mp3name)
		elif opts['encoder'] == 'oggenc':
			if not opts.bin['oggenc']:
				opts.log(1, "'oggenc' binary not found! Aborting track.")
				continue
			encodecmd = "%s %s -o \"%s\" \"%s\"" %\
			  (opts.bin['oggenc'], opts['encopts'], mp3name, wavname)
		else:
			encodecmd = ''

		# Set up Normalizer 
		if opts.bin['normalize-audio']:
			normcmd = '%s "%s" ' % (opts.bin['normalize-audio'], wavname)
		else:
			normcmd = ''
			if not opts['nonormal']:
				opts.log(3, "'normalize-audio' binary not found, skipping pass.")

		# If the target file exists, don't do it, unless we have a mandate.
		if os.path.exists(mp3name) and not opts['force']:
			opts.log(1, "Skipping existing file: %s\n" % mp3name)
			continue
		
		# Be quiet if asked
		if opts['verbosity'] == 0:
			decodecmd += " >/dev/null 2>&1"
			normcmd += " >/dev/null 2>&1"
			if encodecmd:
				encodecmd += " >/dev/null 2>&1"

		# Check for enough tempspace for wav
		dir = os.path.dirname(wavname)
		need = os.stat(filename).st_size * 10 # Require 10x encoded file size
		free = os.statvfs(dir).f_bfree * os.statvfs(dir).f_frsize
		if need > free:
			opts.log(1, "Not enough free space in %s to decode %s, skipping." % \
			  (dir, os.path.basename(filename)))
			continue

		# Actually read and write files:
		# Decode
		opts.log(2, "Decoding...")
		ret = os.system(deunicode(decodecmd))
		if ret != 0:
			opts.log(1, "Error decoding file %s, skipping file." % filename)
			if os.path.exists(wavname):
				os.unlink(wavname)
			continue
		if (bytecount + (os.stat(wavname).st_size/opts['compfactor'])) > opts['maxsize']:
			opts.log(1, "Size limit reached - aborting: %s" % filename)
			os.unlink(wavname)
			return 1

		# Normalize
		if not opts['nonormal'] and normcmd:
			opts.log(2, "Normalizing...")
			ret = os.system(normcmd)
			if ret != 0:
				opts.log(1, "Error normalizing %s, attempting to continue" % wavname) 

		# Encode
		opts.log(2, "Encoding...")
		if opts['encoder'] != 'wave':
			opts.log(1, '')
			ret = os.system(deunicode(encodecmd))
			#if wavname != mp3name:
			os.unlink(wavname)
			if ret != 0:
				opts.log(1, "Error encoding file %s, skipping file." % wavname)
				if os.path.exists(mp3name):
					os.unlink(mp3name)
				continue
		bytecount += os.stat(mp3name).st_size

		# Tag
		opts.log(2, 'Tagging...')
		if opts['encoder'] == 'lame':
			out = ID3.ID3(mp3name, as_tuple=1)
		elif opts['encoder'] == 'oggenc':
			vf = ogg.vorbis.VorbisFile(mp3name)
			out = vf.comment()
		else:
			out = {}

		out['TITLE'] = tags['TITLE'][0]
		out['ARTIST'] = tags['ARTIST'][0]
		out['ALBUM'] = tags['ALBUM'][0]
		if tags.has_key('TRACKNUMBER'):
			out['TRACKNUMBER'] = tags['TRACKNUMBER'][0]
		if opts['brwarning']:
			out['COMMENT'] = opts['brwarning']

		if opts['encoder'] == 'lame':
			out.write()
			del out
		elif opts['encoder'] == 'oggenc':
			out.write_to(mp3name)
			del out
			del vf
		if opts['index']:
			try:
				ifile = file(indexname, 'a')
				ifile.write(mp3name + "\n")
				for k in out:
					ifile.write("%s: %s\n" % (k, out[k]))
				ifile.write("\n\n")
				ifile.close()
			except IOError:
				opts.log(1, "Can't open index file %s" % indexname)

		opts.log(1, "\n")



def deunicode(str):
	if type(str) is type(u""):
		str = str.encode('ascii', 'replace')
	str = str.replace('\0', '')
	return str

