#!/usr/bin/python

#  - main.py - 
# This file is part of mp3togo

# Convert audio files to play on a mp3 player
#
# (c) Simeon Veldstra 2004, 2006  <reallifesim@gmail.com>
#
# This software is free.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You may redistribute this program under the terms of the
# GNU General Public Licence version 2
# Available in this package or at http://www.fsf.org


"""Convert a list of files.

	Find a list of audio files in various 
	formats and qualities and convert them
	to a uniform format suitable for a 
	portable player."""

import sys
import os
import converter
import setup


def fail(mesg, code=1):
	if mesg:
		print >>sys.stderr, mesg
	sys.exit(code)

def main():
	# Read in configuration
	try:
		opts = setup.Options()
		opts.getconf(argv=sys.argv)
	except setup.Fail, msg:
		print setup.Options().usage()
		fail(str(msg))

	# Check for sanity
	if not opts.bin[opts['encoder']]:
		opts.log(1, "Encoder binary '%s' not found in PATH! Aborting." % opts['encoder'])
		fail('')
	if not opts.bin['normalize-audio'] and not opts['nonormal']:
		opts.log(1, "'normalize-audio' binary not found. Normalization disabled.")
		opts['nonormal'] = True

	# Build a playlist
	playlist = []
	def readlist(list, playlist):
		try:
			plist = file(list, 'r')
			playlist += plist.readlines()
			plist.close()
		except:
			opts.log(1, "Can't open playlist: %s" % list)
	if opts['playlist']:
		readlist(opts['playlist'], playlist)
	if opts['arg_files']:
		playlist += opts['arg_files']
	if opts['readxmms'] and opts.bin['xmms-shell']:
		listname = "mp3togo-playlist." + str(os.getpid())
		listname = os.path.join(opts['tempdir'], listname)
		cmd = '%s -e "save %s"' % (opts.bin['xmms-shell'], listname)
		os.system(cmd)
		if os.path.exists(listname):
			readlist(listname, playlist)
			os.unlink(listname)
	elif opts['readxmms'] and not opts.bin['xmms-shell']:
		opts.log(1, "xmms-shell binary not found in path: aborting.")
		fail('')
	cleanlist = []
	for line in playlist:
		if line.startswith('#'):                  #.m3u files, comments
			continue
		if line[-1:] == "\n":
			line = line[:-1]
		if line[:4] == "File" and "=" in line:    #XMMS .pls files
			line = "=".join(line.split("=")[1:])
		if not os.path.exists(line):
			opts.log(2, "\"%s\" is not a valid file" % line)
			continue
		cleanlist.append(line)
	playlist = cleanlist

	# Check for a destination
	if not os.path.isdir(opts['playerdir']):
		opts.log(1, "target_dir does not exist")
		fail('')
	
	# Calculate available space
	fsp = os.statvfs(opts['playerdir']).f_bfree * os.statvfs(opts['playerdir']).f_frsize
	if fsp and (opts['maxsize'] == 0 or opts['maxsize'] > fsp):
		opts['maxsize'] = fsp

	# Offer a hint
	if len(playlist) == 0:
		print opts.usage()
		opts.log(1, "No files to convert.")
		fail('')

	# Start work
	converter.eatlist(opts, playlist)

if __name__ == "__main__": main()

